@@ -175,7 +175,7 @@ WRITERS = {
 }
 
 
-FALSY = ("False", "false", "No", "no", "0", False, 0)
+FALSY = ("False", "false", "No", "no", "0", False, 0, "None")
 TRUTHY = ("True", "true", "Yes", "yes", "1", True, 1)
 
 
@@ -394,6 +394,15 @@ def appendstr(col, s, newcol=None):
     pipeout(df)
 
 
+@register
+def split(col, pat=" "):
+    df = pipein()
+    if col not in df.columns:
+        exit("No such column {}".format(col))
+    df[[col, col + "_rhs"]] = df[col].str.split(pat=pat, n=1, expand=True)
+    pipeout(df)
+
+
 @register
 def astype(type, column=None, newcolumn=None):
     """Cast a column to a different type.
@@ -475,11 +484,13 @@ def crosstab(column):
 
 
 @register
-def groupby(*columns, how="sum", as_index=True):
+def groupby(*columns, how="sum", as_index=False):
     """Group by columns, then apply `how` function.
 
-    Usage: cat a.csv | ph groupby animal --how=mean
-           cat a.csv | ph groupby animal --how=mean --as_index=False
+    Usage: cat a.csv | ph groupby animal  # default to sum
+           cat a.csv | ph groupby animal --how=mean
+           cat a.csv | ph groupby animal --how=prod
+           cat a.csv | ph groupby animal --as_index=True  # removes index
     """
     columns = list(columns)
     if not columns:
@@ -505,6 +516,135 @@ def groupby(*columns, how="sum", as_index=True):
     pipeout(retval)
 
 
+@register
+def rolling(window, how="sum", win_type=None, std=None, beta=None, tau=None):
+    """Rolling window calculations using provided `how` function.
+
+    Usage: cat a.csv | ph rolling 3
+           cat a.csv | ph rolling 5 --win_type=gaussian --std=7.62
+    """
+    df = pipein()
+
+    rollin = df.rolling(window, win_type=win_type)
+    try:
+        fn = getattr(rollin, how)
+    except AttributeError:
+        exit("Unknown --how={}, should be sum, mean, ...".format(how))
+
+    if {std, beta, tau} != {None}:
+        retval = fn(std=std, beta=beta, tau=tau)
+    else:
+        retval = fn()
+
+    pipeout(retval)
+
+
+@register
+def ewm(
+    min_periods=0,
+    adjust=True,
+    ignore_na=False,
+    axis=0,
+    com=None,
+    span=None,
+    halflife=None,
+    alpha=None,
+    how="mean",
+):
+    """Provide exponential weighted functions.
+
+    A related set of functions are exponentially weighted versions of
+    several of the above statistics. A similar interface to rolling and
+    expanding is accessed through the ewm method to receive an EWM
+    object.  A number of expanding EW (exponentially weighted) methods
+    are provided:
+
+      * mean
+      * var
+      * std
+      * corr
+      * cov
+
+    Usage: cat a.csv | ph ewm --com=0.5 --how=mean
+           cat a.csv | ph ewm --halflife=0.5 --how=std
+
+    """
+    if {com, span, halflife, alpha} == {None}:
+        exit("Must pass one of com, span, halflife, or alpha")
+
+    df = pipein()
+
+    ewm_ = df.ewm(
+        min_periods=min_periods,
+        adjust=adjust,
+        ignore_na=ignore_na,
+        axis=axis,
+        com=com,
+        span=span,
+        halflife=halflife,
+        alpha=alpha,
+    )
+    try:
+        fn = getattr(ewm_, how)
+    except AttributeError:
+        exit("Unknown --how={}, should be mean, var, std, corr, cov..".format(how))
+
+    retval = fn()
+
+    pipeout(retval)
+
+
+@register
+def expanding(min_periods=1, center=False, axis=0, how="sum", quantile=None):
+    """Provide expanding transformations.
+
+    A common alternative to rolling statistics is to use an expanding
+    window, which yields the value of the statistic with all the data
+    available up to that point in time.
+
+    For working with data, a number of window functions are provided for
+    computing common window or rolling statistics.  Among these are
+    count, sum, mean, median, correlation, variance, covariance,
+    standard deviation, skewness, and kurtosis.
+
+
+    Usage: cat a.csv | ph expanding
+           cat a.csv | ph expanding 1 --how=sum   # above equivalent to this
+           cat a.csv | ph expanding 2
+           cat a.csv | ph expanding 5 --how=quantile --quantile=0.25
+           cat a.csv | ph expanding 3 --center=True
+
+    """
+
+    df = pipein()
+
+    if center in TRUTHY:
+        center = True
+    elif center in FALSY:
+        center = False
+    else:
+        exit("center must be True of False, not {}".format(center))
+
+    if quantile is not None:
+        if how != "quantile":
+            exit("Use both or none of --how=quantile and --quantile=<float>")
+    if how == "quantile" and quantile is None:
+
+        exit("--how=quantile needs --quantile=<float>, e.g. --quantile=0.25")
+    expanding_ = df.expanding(min_periods=min_periods, center=center, axis=axis)
+    try:
+        fn = getattr(expanding_, how)
+    except AttributeError:
+        exit("Unknown --how={}, should be sum, mean, max, quantile..".format(how))
+
+    if how == "quantile":
+        retval = fn(quantile)
+    else:
+        retval = fn()
+
+    pipeout(retval)
+
+
 @register
 def monotonic(column, direction="+"):
     """Check if a certain column is monotonically increasing or decreasing.
@@ -754,6 +894,7 @@ def from_(ftype="csv", **kwargs):
     """Read a certain (default csv) format from standard in and stream out as csv.
 
     Usage: cat a.json | ph from json
+           cat /etc/passwd | ph from csv --sep=':' --header=None
 
     The following pipes should be equivalent:
 
@@ -763,8 +904,11 @@ def from_(ftype="csv", **kwargs):
     cat a.tsv | ph from csv --sep='\t'
     cat a.tsv | ph from csv --sep='\t' --thousands=','
 
-
+    In the event that the csv data starts on the first line (i.e. no
+    header is present), use --header=None.
     """
+    if "header" in kwargs:
+        kwargs["header"] = __tryparse(kwargs["header"])
     skiprows = kwargs.get("skiprows")
     if skiprows is not None:
         try:
@@ -973,7 +1117,13 @@ def open_(ftype, fname, **kwargs):
            ph open csv a.csv --thousands=','
 
 
+    In the event that the csv data starts on the first line (i.e. no
+    header is present), use --header=None.
+
     """
+    if "header" in kwargs:
+        kwargs["header"] = __tryparse(kwargs["header"])
+
     if ftype not in READERS:
         exit("Unknown filetype {}".format(ftype))
     reader = READERS[ftype]
@@ -1048,7 +1198,11 @@ def _call(attr, *args, **kwargs):
 
 
 def register_forward(attr):
-    COMMANDS[attr] = lambda: _call(attr)
+    def partial(*args, **kwargs):
+        return _call(attr, *args, **kwargs)
+
+    partial.__name__ = attr
+    COMMANDS[attr] = partial
 
 
 @register
@@ -1078,21 +1232,26 @@ def tail(n=10):
 
 
 def __tryparse(x):
+    if x is None or x == "None":
+        return None
     x_ = x
     try:
         x_ = float(x)
         x_ = int(x)
     except ValueError:
         pass
+    except OverflowError:
+        x_ = float("inf")
     return x_
 
 
 @register
-def replace(column, old, new, newcolumn=None):
-    """Replace a value in a column with a new value.
+def replace(old, new, column=None, newcolumn=None):
+    """Replace a value (in a column) with a new value.
 
-    Usage: cat a.csv | ph replace y 8 100
-           cat a.csv | ph replace y 8 100 z
+    Usage: cat a.csv | ph replace 8 100 # replace in all columns
+           cat a.csv | ph replace 8 100 --column=y
+           cat a.csv | ph replace 8 100 --column=y --newcolumn=z
 
     Beware that it is difficult to know which _types_ we are searching for,
     therefore we only apply a heuristic, which is doomed to be faulty.
@@ -1100,12 +1259,15 @@ def replace(column, old, new, newcolumn=None):
     if newcolumn is None:
         newcolumn = column
     df = pipein()
-    if df[column].dtype != object:
-        old = __tryparse(old)
-        new = __tryparse(new)
-    if column not in df:
+
+    if column is None:
+        if newcolumn is not None:
+            exit("Cannot use newcolumn and not column.")
+        df = df.replace(to_replace=old, value=new, inplace=False)
+    elif column not in df:
         exit("Column {} does not exist.".format(column))
-    df[newcolumn] = df[column].replace(to_replace=old, value=new, inplace=False)
+    else:
+        df[newcolumn] = df[column].replace(to_replace=old, value=new, inplace=False)
     pipeout(df)
 
 
@@ -1311,7 +1473,7 @@ def _main(argv):
             split = a.index("=")
             k = a[2:split]
             v = a[split + 1 :]
-            kwarg[k] = v
+            kwarg[k] = __tryparse(v)
         else:
             args.append(__tryparse(a))
     try:
